/// <reference types="node" />
import { EventEmitter } from 'events';
declare type MigrationHistory = import('./migration').MigrationHistory;
declare type MigrationRecord = import('./migration').MigrationRecord;
declare type MigrationSource = import('./migration').MigrationSource;
declare type MigrationVersion = import('./migration').MigrationVersion;
declare type SynorConfig = import('..').SynorConfig;
declare type MigratorEventStore = {
    'lock:start': [];
    'lock:end': [];
    'unlock:start': [];
    'unlock:end': [];
    'open:start': [];
    'open:end': [];
    'close:start': [];
    'close:end': [];
    'drop:start': [];
    'drop:end': [];
    'version:start': [];
    version: [MigrationVersion];
    'version:end': [];
    'history:start': [];
    history: [MigrationHistory];
    'history:end': [];
    'pending:start': [];
    pending: [MigrationSource[]];
    'pending:end': [];
    'validate:start': [];
    'validate:run:start': [MigrationRecord];
    'validate:error': [MigrationRecord, Error];
    'validate:run:end': [MigrationRecord];
    'validate:end': [];
    'migrate:start': [];
    'migrate:run:start': [MigrationSource];
    'migrate:error': [MigrationSource, Error];
    'migrate:run:end': [MigrationSource];
    'migrate:end': [];
    'repair:start': [];
    'repair:end': [];
    error: [Error];
};
declare type EventStore = Record<string | symbol, any[]>;
declare type Listener<S extends EventStore, N extends keyof S> = (...data: S[N]) => void;
declare type UpdateListener<S extends EventStore, R extends any> = <N extends keyof S>(event: N, listener: Listener<S, N>) => R;
export interface SynorMigrator extends EventEmitter {
    addListener: UpdateListener<MigratorEventStore, this>;
    on: UpdateListener<MigratorEventStore, this>;
    once: UpdateListener<MigratorEventStore, this>;
    prependListener: UpdateListener<MigratorEventStore, this>;
    prependOnceListener: UpdateListener<MigratorEventStore, this>;
    removeListener: UpdateListener<MigratorEventStore, this>;
    off: UpdateListener<MigratorEventStore, this>;
    removeAllListeners(event?: keyof MigratorEventStore): this;
    listeners<N extends keyof MigratorEventStore>(event: N): Array<Listener<MigratorEventStore, N>>;
    rawListeners<N extends keyof MigratorEventStore>(event: N): Array<Listener<MigratorEventStore, N>>;
    emit<N extends keyof MigratorEventStore>(event: N, ...data: MigratorEventStore[N]): boolean;
    listenerCount(type: keyof MigratorEventStore): number;
}
export declare class SynorMigrator extends EventEmitter {
    private readonly config;
    private readonly database;
    private readonly source;
    private locked;
    constructor(config: SynorConfig);
    private readonly decorate;
    private readonly emitOrThrow;
    private readonly lock;
    private readonly unlock;
    open: () => Promise<void>;
    close: () => Promise<void>;
    drop: () => Promise<void>;
    version: () => Promise<void>;
    history: (recordStartId?: number) => Promise<void>;
    pending: () => Promise<void>;
    validate: () => Promise<void>;
    migrate: (targetVersion: string) => Promise<void>;
    repair: () => Promise<void>;
}
export {};
