declare type Connection = import('mysql').Connection;
declare type MigrationRecord = import('../../core/migration').MigrationRecord;
declare type ColumnName = 'version' | 'type' | 'title' | 'hash' | 'applied_at' | 'applied_by' | 'execution_time' | 'dirty';
declare type LockResult = 0 | 1 | null;
export declare type QueryStore = {
    openConnection: () => Promise<void>;
    closeConnection: () => Promise<void>;
    getMigrationTableColumnNames: () => Promise<string[]>;
    createMigrationTable: () => Promise<void>;
    addColumn: Record<ColumnName, () => Promise<void>>;
    getLock: () => Promise<LockResult>;
    releaseLock: () => Promise<LockResult>;
    getTableNames: () => Promise<string[]>;
    dropTables: (tableNames: string[]) => Promise<void>;
    getRecords: (startId?: number) => Promise<MigrationRecord[]>;
    addRecord: (record: Omit<MigrationRecord, 'id'>) => Promise<void>;
    deleteDirtyRecords: () => Promise<void>;
    updateRecord: (id: MigrationRecord['id'], data: Pick<MigrationRecord, 'hash'>) => Promise<void>;
};
declare type QueryStoreOptions = {
    migrationTableName: string;
    databaseName: string;
    advisoryLockId: string;
};
declare type QueryValue = boolean | number | string | Date;
export declare function runQuery<T = any>(connection: Connection, query: string, values?: QueryValue[]): Promise<T>;
export declare function getQueryStore(connection: Connection, { migrationTableName: tableName, databaseName, advisoryLockId }: QueryStoreOptions): QueryStore;
export {};
