'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var fs = require('fs');
var path = require('path');
var util = require('util');

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _toArray(arr) {
  return _arrayWithHoles(arr) || _iterableToArray(arr) || _nonIterableRest();
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread();
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  }
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArray(iter) {
  if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter);
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance");
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

var smallerThan = {
  ASC: -1,
  DESC: 1
};
var greaterThan = {
  ASC: -1,
  DESC: 1
};

function compareVersion() {
  var direction = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'ASC';
  var versionA = arguments.length > 1 ? arguments[1] : undefined;
  var versionB = arguments.length > 2 ? arguments[2] : undefined;

  if (versionA < versionB) {
    return smallerThan[direction];
  }

  if (versionA > versionB) {
    return greaterThan[direction];
  }

  return 0;
}

function sortVersions(_ref) {
  var _ref2 = _toArray(_ref),
      versions = _ref2.slice(0);

  var compareFn = compareVersion.bind(null, 'ASC');
  return versions.sort(compareFn);
}

function _await(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

var readFile = util.promisify(fs.readFile);

function _async(f) {
  return function () {
    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

var readDir = util.promisify(fs.readdir);
var FileSourceEngine = function FileSourceEngine(uri, _ref) {
  var migrationInfoParser = _ref.migrationInfoParser;

  var _ref2 = new URL(uri),
      sourcePath = _ref2.pathname;

  var migrationsByVersion = {};
  var sortedVersions = [];

  var open = _async(function () {
    return _await(readDir(sourcePath, {
      withFileTypes: true
    }).then(function (entries) {
      return entries.filter(function (entry) {
        return entry.isFile();
      });
    }), function (entries) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = entries[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var entry = _step.value;
          var migrationInfo = migrationInfoParser(entry.name);
          migrationsByVersion[migrationInfo.version] = Object.assign({}, migrationsByVersion[migrationInfo.version], _defineProperty({}, migrationInfo.type, migrationInfo));
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator["return"] != null) {
            _iterator["return"]();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      var versions = sortVersions(Object.keys(migrationsByVersion));
      sortedVersions.push.apply(sortedVersions, _toConsumableArray(versions));
    });
  });

  var close = function close() {
    return _await();
  };

  var first = _async(function () {
    var version = sortedVersions[0];
    return version || null;
  });

  var prev = _async(function (version) {
    var index = sortedVersions.indexOf(version);
    var exists = index !== -1;
    var first = index === 0;

    if (first || !exists) {
      return null;
    }

    var prevVersion = sortedVersions[index - 1];
    return prevVersion;
  });

  var next = _async(function (version) {
    var index = sortedVersions.indexOf(version);
    var exists = index !== -1;
    var last = index === sortedVersions.length - 1;

    if (last || !exists) {
      return null;
    }

    var nextVersion = sortedVersions[index + 1];
    return nextVersion;
  });

  var last = _async(function () {
    var version = sortedVersions[sortedVersions.length - 1];
    return version || null;
  });

  var read = _async(function (_ref3) {
    var filename = _ref3.filename;
    var migrationFilePath = path.join(sourcePath, filename);
    return readFile(migrationFilePath);
  });

  var get = _async(function (version, type) {
    var migrations = migrationsByVersion[version];

    if (!migrations) {
      return null;
    }

    var migrationInfo = migrations[type];
    return migrationInfo ? migrationInfo : null;
  });

  return {
    path: sourcePath,
    open: open,
    close: close,
    first: first,
    prev: prev,
    next: next,
    last: last,
    get: get,
    read: read
  };
};

exports.FileSourceEngine = FileSourceEngine;
